//
//  ChickenDocument.m
//  ChickenApp
//
//  Created by Kon Lovett on 6/4/06.
//  Copyright 2006 YourSelf Enterprises. All rights reserved.
//

#import "ChickenDocument.h"

@implementation ChickenDocument

//
// PRIVATE
//

- (NSString *)text
{
    return [textView string];
}

- (void)setText:(NSString *)str
{
    [textView setString: str];
}

- (NSString *)selectedText
{
    NSRange rng = [textView selectedRange];
    if (0 < rng.length) {
        return [[textView string] substringWithRange: rng];
    }
    return nil;
}

- (void)scrollToBottom
{
	NSRange rng;
	[textView selectAll: self];
	rng = [textView selectedRange];
	rng.location = rng.length;
	rng.length = 0;
	[textView scrollRangeToVisible: rng];
	[textView setSelectedRange: rng];
}

- (void)appendText:(NSString *)str
{
    NSAttributedString *atrstr = [[NSAttributedString allocWithZone:[self zone]] initWithString: str];
    [[textView textStorage] appendAttributedString: atrstr];
    [atrstr release];
    [[textView textStorage] appendAttributedString: tapeSeparator];
	[self scrollToBottom];
}

//
// PUBLIC
//

- (id)init
{
  if (self = [super init]) {
	  tapeSeparator = [[NSAttributedString allocWithZone:[self zone]] initWithString: @"\n"];
  }
  return self;
}

- (void)dealloc
{
    [csiProcess stopProcess];
    [csiProcess release];
	[tapeSeparator release];
    [super dealloc];
}

- (NSString *)windowNibName {
    return @"ChickenDocument";
}

- (void)windowControllerDidLoadNib:(NSWindowController *)cntlr
{
  [super windowControllerDidLoadNib: cntlr];

  [textView setFont: [NSFont fontWithName: @"Monaco" size: 12.0]];
  
  csiProcess = [[CSIProcess allocWithZone:[self zone]] initWithController: self];
  [csiProcess startProcess];
}

// Deprecated for 10.4
- (NSData *)dataRepresentationOfType:(NSString *)type {
    return [[self text] dataUsingEncoding: NSUTF8StringEncoding];
}

// Deprecated for 10.4
- (BOOL)loadDataRepresentation:(NSData *)data ofType:(NSString *)type {
    NSString *str = [[NSString allocWithZone:[self zone]] initWithData: data encoding: NSUTF8StringEncoding];
    if (str) {
        [self setText: str];
		[str release];
        return YES;
    }
    return NO;
}

#if 0
- (BOOL)validateUserInterfaceItem:(id <NSValidatedUserInterfaceItem>)ui
{
    return YES;
}

// Deprecated for 10.4
- (BOOL)validateMenuItem:(NSMenuItem *)mi
{
	return YES;
}
#endif

- (IBAction)evaluateSelection:(id)sender
{
    NSString *str = [self selectedText];
    if (str) {
        [csiProcess performExpression: str];
    }
}

- (IBAction)systemInformation:(id)sender
{
	[csiProcess reportSystemInfoAction];
}

//
// <CSIProcessController>
//

- (void)csiProcessResultDictionary:(NSDictionary *)outputs forCommand:(CSICommand *)cmd
{
	NSEnumerator *enumerator = [outputs keyEnumerator];
	id key;
	
	while ((key = [enumerator nextObject])) {
		[self appendText: [outputs objectForKey: key]];
	}
}

- (void)csiProcessResult:(NSString *)output
{
    [self appendText: output];
}

- (void)csiProcessError:(NSString *)output
{
    [self appendText: output];
}

- (void)csiProcessStarted
{
}

- (void)csiProcessFinished
{
}

@end
